%% Processing heart rate using sliding autocorrelection for AAS conference paper
% Carl Howard 12 August 2018


%% Heart Signal definitions and limitations
% say that the healthy, expected BPM is between 100 to 200 BPM
% which means the period between beats is t_shift = 60/200 to 60/100 sec
BPM_low=100;
BPM_high=200;


%% Read in the audio file
[y,Fs]=audioread('heart_beat_filtered_8kHz.wav');

%% Band pass filter the data
% This will  band pass filter the date from 50 to 200 Hz
d = designfilt('bandpassiir','FilterOrder',20, ...
                'HalfPowerFrequency1',50,'HalfPowerFrequency2',200, ...
                'SampleRate',Fs);


% Uncomment the following line if you want to see the filter response
%fvt = fvtool(d,'Fs',Fs);
%x_Fs_44kHz_upto_200Hz=filtfilt(d,x);

% Just work with the first xx seconds of audio
% t_max=length(x_Fs_44kHz_upto_200Hz)/Fs;

y_BP_15_200Hz=filtfilt(d,y);


%% Downsample to 200 Hz
% Now resample to a lower frequency
% from the original 8 kHz down to about 200 Hz.
% The resampling has to be an integer downsample from 8 kHz,
% Hence 8000 / 40 = 200 Hz, which is okay.
fs200=200;
y_200Hz=resample(y_BP_15_200Hz,fs200,Fs);

% Calculate an associated time vector
t_fs200_Hz = [1:(length(y_200Hz))]/(fs200);



% Calculate the envelope using a Hilbert transform
figure
[up_envelope,~] = envelope(y_200Hz*10,5,'peak');
plot(t_fs200_Hz,y_200Hz*10,t_fs200_Hz,up_envelope);
xlabel('Time (s)');
ylabel('Audio')

%%% Only calculate 5 seconds initially to see if the SST is working properly
%up_envelope2=up_envelope(1:fs200*5);



%% Autocorrelation method by CQH
% This method uses a sliding window of the data of 6 seconds and increments
% the analysis by 0.5 s. It uses the envelope signal and the
% auto-correlation function.

sliding_window_length = 2;      %[s] length of window for analysis
time_increment = 0.1;   %[s] increment window in time for next analysis

% Can only do an integer number of analyses, based on the length of time of
% the available data.
t_auto_correl=[0:time_increment:floor(length(up_envelope)/fs200)-sliding_window_length];

number_of_analyses = length(t_auto_correl);

% num_window_samples = round(WINDOW_SECONDS * fps);
% bpm_sampling_period_samples = round(BPM_SAMPLING_PERIOD * fps);
% %num_bpm_samples = floor((size(y, 2) - num_window_samples) / bpm_sampling_period_samples);
% num_bpm_samples = floor((length(y) - num_window_samples) / bpm_sampling_period_samples);


sliding_window_samples=sliding_window_length*fs200;


% Create an empty vector to hold the data
av_BPM_auto_correl=zeros(number_of_analyses,1);

max_lag=60/BPM_low;
max_lag=floor(max_lag*fs200);

for ii=1:number_of_analyses
    % Fill sliding window with original signal
    index_start = (ii-1)*time_increment*fs200+1;
    index_end = index_start+sliding_window_samples-1;
    % Use Hanning window to bring edges to zero. In this way, no artificial
    % high frequencies appear when the signal is treated as periodic by the
    %y = ynw .* hann(size(ynw, 2))';
%     data_window = up_envelope(index_start:index_end) .*   ...
%                 hanning(sliding_window_samples);
    data_window = up_envelope(index_start:index_end) .*   ...
                hanning(sliding_window_samples);
    [auto_corre1_data,lags]=xcorr(data_window,max_lag);
    auto_corre1_norm=auto_corre1_data(max_lag+1:2*max_lag+1)'/auto_corre1_data(max_lag+1);   
    lags2=lags(max_lag+1:2*max_lag+1);

    % Find the peaks in the auto-correlation plot
    [pks,locs] = findpeaks(auto_corre1_norm,fs200);
    [~,index]=max(pks);
    if ~isempty(index)
        av_BPM_auto_correl(ii)=60/locs(index);
    else
        av_BPM_auto_correl(ii)=0;    
    end;
end;



%%
% Plot the Heart Rate vs time that was calculated using the sliding window
% and the auto-correlation functoin
figure
time_shift=1.0;     % This was tweaked to make it line up
p3b=plot(t_auto_correl+time_shift,av_BPM_auto_correl);
axis([0 max(t_auto_correl) BPM_low BPM_high])
xlabel('Time (s)')
ylabel('Heart Rate (BPM)')
title('Heart Rate Calculated Using Auto-Correlation')
axis([0 360 BPM_low BPM_high]);



%% This is the data of manually counting the pulses from the envelope and the raw time size
% the first column is the time where the peak of S1 sound occurs,
% the second column is the BPM, which is calculated as 60/(t_{i}-t_{i-1})
manual_hr2=[0.785	151.8987342
1.17	155.8441558
1.56	153.8461538
1.985	141.1764706
2.405	142.8571429
2.835	139.5348837
3.245	146.3414634
3.62	160
4.02	150
4.4	157.8947368
4.775	160
5.165	153.8461538
5.565	150
5.965	150
6.38	144.5783133
6.795	144.5783133
7.205	146.3414634
7.625	142.8571429
8.005	157.8947368
8.4	151.8987342
8.805	148.1481481
9.215	146.3414634
9.61	151.8987342
10.02	146.3414634
10.44	142.8571429
10.87	139.5348837
11.29	142.8571429
11.71	142.8571429
12.12	146.3414634
12.53	146.3414634
12.9	162.1621622
13.3	150
13.71	146.3414634
14.13	142.8571429
14.54	146.3414634
14.96	142.8571429
15.39	139.5348837
15.83	136.3636364
16.25	142.8571429
16.64	153.8461538
17.05	146.3414634
17.46	146.3414634
17.87	146.3414634
18.27	150
18.69	142.8571429
19.13	136.3636364
19.54	146.3414634
19.98	136.3636364
20.42	136.3636364
];

hold on
pz1=plot(manual_hr2(:,1),manual_hr2(:,2),'--');
legend('Sliding Auto-Correlation','Manual counting')